Bumblebee.ModelSpec behaviour (Bumblebee v0.7.0)
View SourceAn interface for configuring and building models based on the same architecture.
Every module implementing this behaviour is expected to also define a configuration struct.
Summary
Types
@type t() :: Bumblebee.Configurable.t()